@@ -37,12 +37,13 @@ use constant {
     TOP_LABEL_HEIGHT    => 2,
 };
 
-my $VERSION = "0.98";
+my $VERSION = "0.99";
 
 my %opts = (
     cache_dir   => "/var/lib/sysv-rc-conf",
     order       => 'a',
     priority    => '',
+    purge_cache => 0,
     root        => '/',
     show        => '',
     verbose     => '',
@@ -57,6 +58,7 @@ GetOptions("cache=s"	=> \$opts{cache_dir},
            "list:s"     => \$opts{chkcfg_list},
 	   "order=s"	=> \$opts{order},
 	   "priority"	=> \$opts{priority},
+           "Purge"      => \$opts{purge_cache},
 	   "root=s"	=> \$opts{root},
 	   "show=s"	=> \$opts{show},
 	   "verbose=s"	=> \$opts{verbose},
@@ -392,10 +394,15 @@ sub simple_box_changed
     $userdata->{changed}++;
 
     if ($box->get()) {
-	update_link($userdata->{sn}, $userdata->{runlevel}, 'S', undef) 
+	update_link($userdata->{sn}, $userdata->{runlevel}, 'S', undef);
     } 
     else {
-	update_link($userdata->{sn}, $userdata->{runlevel}, 'K', undef)
+        if ($userdata->{initial_state} == 1) {
+            update_link($userdata->{sn}, $userdata->{runlevel}, 'K', undef);
+        }
+        else {
+            update_link($userdata->{sn}, $userdata->{runlevel}, '', undef);
+        }
     }
 }
 
@@ -406,7 +413,12 @@ sub runlevel_status
     opendir (INITD, $initd) or die "$0: opendir $initd : $!";
     while ( defined(my $service = readdir INITD) ) {
 	next if $service =~ /\.sh$/; # see the debian policy manual
+        next if $service =~ /\.dpkg-/; # see the debian policy manual
 	next if $service =~ /^\.+$/; # ignore . and ..
+        next if $service =~ /^rc$/; # ignore rc
+        next if $service =~ /^rcS$/; # ignore rcS
+        next if $service =~ /^skeleton$/; # ignore the sample init.d file
+        next unless -f "$initd/$service"; # ignore non-files
 	next unless -x "$initd/$service"; # ignore stuff like README
 	$runlevels{$service} = { };
     }
@@ -429,7 +441,7 @@ sub runlevel_status
 	closedir DIR or die "$0: closedir $etc_rc$rl.d : $!";
     }
 
-    update_cache(\%runlevels);
+    update_cache(\%runlevels) if ! defined $opts{chkcfg_list};
     return %runlevels;
 }
 
@@ -440,6 +452,14 @@ sub setup_cache_env
         mkdir $opts{cache_dir} or die "Can't create $opts{cache_dir} : $!";
     }
 
+    if ($opts{purge_cache}) {
+        verbose("Purging the services cache");
+        if (-e "$opts{cache_dir}/services") {
+            unlink "$opts{cache_dir}/services"
+                or die "Can't unlink $opts{cache_dir}/services : $!";
+        }
+    }
+
     unless (-e "$opts{cache_dir}/services") {
         # Later we need to open the file with +< which can't create a new file
         # so we'll emulate touch.
@@ -715,10 +735,10 @@ Quick key reference:
   Arrow keys: Move around
   CTRL-n:     Next Page
   CTRL-p:     Previous Page
-  r:          Restore all symlinks back to original state
-  -:          Stop service
-  = or +:     Start service
-  h:          Toggle help on / off
+  r:          Restore all symlinks back to state when sysv-rc-conf started
+  -:          Stop service now
+  = or +:     Start service now
+  h:          Toggle help screen on / off
   q:          Quit
 
   Checkbox Layout:
@@ -730,9 +750,14 @@ Quick key reference:
     CTRL-f:     Move cursor forward through text
     CTRL-b:     Move cursor backwards through text
 
-  See the man page for information on how the system uses the init 
-  script links, how to get new version of the program, how to submit 
-  bug reports, etc.  
+  Note: When using either GUI layout (checkbox or priority), all 
+  configuration changes to the symlinks will happen immediately, not 
+  when the program exits.
+
+  See the man page for much more detailed documentation on how to use
+  sysv-rc-conf and it's various options, also it documents how the system
+  uses the init script links, how to get new version of the program, how to
+  submit bug reports, etc.
 
   sysv-rc-conf is released under the GNU GPL.
   Version: $VERSION
@@ -766,9 +791,14 @@ sub draw_simple_layout
 
     for (my $i = 0, my $right_n = 12; $i <= $#show_rls; $i++, $right_n += 8) {
 	my $on_or_off = 0;
+        my $initial_state = 0;
 	# We only want to show S\d\d services as selected. 
-	$on_or_off = 1 if exists $runlevels{$sn}{$show_rls[$i]} 
-		          &&     $runlevels{$sn}{$show_rls[$i]} =~ /^S\d\d$/;
+        if (exists $runlevels{$sn}{$show_rls[$i]}) {
+            if ($runlevels{$sn}{$show_rls[$i]} =~ /^S\d\d$/) {
+                $on_or_off = 1;
+            }
+            $initial_state = 1;
+        }
 
         my $box = $screen->add(
             zero_pad($row-2).zero_pad($i), 'Checkbox',
@@ -783,6 +813,7 @@ sub draw_simple_layout
 			     sn		=> $sn,
 			     changed	=> 0,
 			     runlevel	=> $show_rls[$i],
+                             initial_state => $initial_state,
 			   },
             -onchange   => \&simple_box_changed,
             -onfocus    => \&got_focus,
@@ -801,6 +832,7 @@ sub draw_priority_layout
 	my $text = exists $runlevels{$sn}{$show_rls[$i]}
 		    ? $runlevels{$sn}{$show_rls[$i]}
 		    : '';
+
 	my $box = $screen->add(
 	    zero_pad($row-2).zero_pad($i), 'TextEntry',
 	    -sbborder   => 1,
@@ -978,6 +1010,8 @@ B<sysv-rc-conf> - Run-level configuration for SysV like init script links
 
 B<sysv-rc-conf> [ I<options> ]
 
+B<sysv-rc-conf>  --list  [ I<service> ]
+
 B<sysv-rc-conf> [ --level I<levels> ] I<service> E<lt>I<on|off>E<gt>
 
 =head1 DESCRIPTION
@@ -989,7 +1023,8 @@ allows for more fine tuned management of the symlinks. It's a replacement for
 programs like B<ntsysv(8)> or B<rcconf(8)>.
 
 B<sysv-rc-conf> can also be used at the command line when the desired changes
-to the symlinks are already known. The syntax is borrowed from B<chkconfig(8)>.
+to the symlinks are already known. The syntax is borrowed from B<chkconfig(8)>,
+although it does not follow it exactly.
 
 =head1 GENERAL OPTIONS
 
@@ -999,13 +1034,18 @@ to the symlinks are already known. The syntax is borrowed from B<chkconfig(8)>.
 
 The directory where the priority numbers, old runlevel configuration, etc. 
 should be stored. This defaults to C</var/lib/sysv-rc-conf>. See the FILES
-section below.
+section below and the --Purge option.
 
 =item B<-r> DIRECTORY, B<--root=>DIRECTORY
 
 The root directory to use. This defaults to C</>. This comes in handy if the
 root file system is mounted somewhere else, such as when using a rescue disk.
 
+=item B<-P>, B<--Purge>
+
+Purge the information stored in the cache file. See the FILES section below
+and the --cache option.
+
 =item B<-v> FILE, B<--verbose=>FILE
 
 Print verbose information to C<FILE>
@@ -1087,14 +1127,19 @@ for that service is displayed.
 
 =head1 USING THE GUI
 
+=head2 Note
+
+When using either GUI layout described below, all configuration 
+changes to the symlinks will happen immediately, not when the program exits.
+
 =head2 Using the Default layout
 
 The default (simple) layout shows in a grid fashion all of the services that
 are in C<init.d> and which runlevels they are turned on at. For example, where
-the C<ssh> row and C<3> column intersect, if there is a checkbox there that
-means the service will be turned on when entering runlevel 3. If there is no
-checkbox it can mean that either there are no links to the service in that
-specific runlevel, or that the service is turned off when entering that 
+the C<ssh> row and C<3> column intersect, if there is an 'X' in the box there
+that means the ssh service will be turned on when entering runlevel 3. If there
+is no checkbox it can mean that either there are no links to the service in
+that specific runlevel, or that the service is turned off when entering that
 runlevel. If more configuration detail is needed, see the next paragraph and
 the B<--priority> option.
 
@@ -1124,6 +1169,7 @@ press the B<r> key.  The B<h> key will display a quick reference help screen.
 =head2 Default layout
 
 When using the default layout use the space bar to toggle the service on / off.
+An 'X' in the checkbox indicates that the service is on.
 
 =head2 Priority layout
 
@@ -1135,8 +1181,8 @@ into the field.
 
 =head2 Starting / Stopping Services
 
-To start a service, press the C<+> or C<=> key.
-To stop  a service, press the C<-> key.
+To start a service now, press the C<+> or C<=> key.
+To stop  a service now, press the C<-> key.
 
 This will call C</etc/init.d/service start> or C</etc/init.d/service stop>.
 
@@ -1161,7 +1207,8 @@ C</etc/rc{runlevel}.d/> in C</var/lib/sysv-rc-conf/services> (See the --cache
 option to change the location of this file). It uses this cache to make an
 intelligent decision on what priority number to give the K or S link when they
 are changed in the simple layout. This cache is updated/created everytime the
-program is launched.  The format of the file is as follows:
+program is launched. The program needs to run with root privileges in
+order to update the cache. The format of the file is as follows:
 
   RUNLEVEL S|K PRIORITY SERVICE
 
@@ -1200,7 +1247,7 @@ C</etc/init.d/README>).
 =head1 CAVEATS
 
 B<sysv-rc-conf> only manages the symlinks in the C<rc{runlevel}.d>
-directories. It's possible that pacakages may have other ways of being
+directories. It's possible that packages may have other ways of being
 disabled or enabled. 
 
 Because Curses takes over the screen sometimes error messages won't be
